# ---------------------------------------------------------------------------------------------------------------------
# Este script 1) Calcula el NDBI contruido con las capas HH Y HV pre-procesadas. Este indice intente
#                identificar la contribucion del volumen de dispersion de diferentes polarizaciones
#             2) Calculo de la textura de las bandas HH y HV asico como del NDBI, que contribuyen capturar 
#                variaciones en la estructura de la vegetacion  
# 
# 
#------------------------------------------------------------------------------------------------------------------------
# PASO 1:  CONFIGURACION DEL DIRECTORIO DE TRABAJO Y LECTURA DE IMAGENES
#--------------------------------------------------------------------------------------------------------------------------

# cargar paquetes utilizados en este modulo
library(glcm)        
library(rgeos)
library(raster)
library(sp)


##### Definir directorio de trabajo
setwd(choose.dir(default = "", 
                 caption = "Seleccionar folder con las imegenes en formato IMG"))
getwd()
folder <- getwd()

# cargar el shapefile del area de estudio
area <- shapefile(choose.files(
  caption = "selecciona el shapefile del área de estudio"))
plot(area)
summary(area)

# transformacion de coordenadas del pologono de interes similar a la proyeccion de los imegenes
newProj <- "+proj=lcc +lat_1=17.5 +lat_2=29.5 +lat_0=12 +lon_0=-102 +x_0=2500000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"

newarea<-  spTransform(area,newProj)
newarea

# Leer los archivos de cada polarizacion
FileListHH <- raster("HH.img")
FileListHH
FileListHV <- raster("HV.img")
FileListHV

# PASO 2: CALCULO DEL INDICE NDBI CON LAS CAPAS HH Y HV
#--------------------------------------------------------------------------------------------------------------------------
# NDBI = (HH - HV)/(HH + HV)
overlay(FileListHH,
        FileListHV,
        # Definir la funcion
        fun=function (x,y) {return((x - y)/(x + y))},
        filename= paste(folder,"/","NDBI", sep = ""),
        overwrite =TRUE,
        format="HFA")

# PASO 3: CALCULO DE LAS MEDIDAS DE TEXTURA
#--------------------------------------------------------------------------------------------------------------------------
# Generar la lista de nombres de archivos a procesar 
FileListIMG <- list.files(path = folder, pattern = ".img$")
FileListIMG

# Crea lista de vectores para almacenar los resultados de las texturas
List <- vector(mode = "list",
               length = length(FileListIMG))
List

# calculo de texturas para todas las .img en la lista
for (i in 1:length(FileListIMG)){
  List[[i]] <- glcm(raster(FileListIMG[i]),
                    n_grey=64,
                    window = c(3,3),
                    shift=list(c(0,1), c(1,1), c(1,0), c(1,-1)),
                    statistics = c("mean", "variance", "homogeneity",
                                   "contrast", "dissimilarity", "entropy",
                                   "second_moment", "correlation"),
                    min_x=NULL, max_x=NULL,
                    na_opt="any", na_val=NA, 
                    scale_factor=1,
                    asinteger=FALSE)
}

#load("Texturas.RData")

for (i in 1:length(List)){
  plot(List[[i]])
}
#crear el directorio de salida de texturas
{Name.Dir.text <- "IMG_texturas"
{ifelse(!
          dir.exists(file.path(folder,Name.Dir.text)),
        dir.create(file.path(folder,Name.Dir.text)), FALSE
)
  dir.text <- file.path(folder,Name.Dir.text)}
}
#save.image(file = "TEXTURAS.RData")

#Guardar las imagenes de textura en formato HFA
for (i in 1:length(FileListIMG)){
  writeRaster(List[[i]]$glcm_mean,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_mean", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_variance,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$", 
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_vari", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_homogeneity,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_homo", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_contrast,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_cont", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_dissimilarity,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_diss", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_entropy,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$", 
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_entr", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_second_moment,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_seco", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_correlation,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_corr", sep = ""),
              overwrite =TRUE,
              format="HFA")
}

# PASO 5: CORRECCION DE ARCCHIVOS
#--------------------------------------------------------------------------------------------------------------------------

#Lista de raster de salida 
FileListIMG_t <- list.files(path = dir.text, pattern = ".img$",
                            full.names = TRUE)
FileListIMG_t

# Lista de textura correlation para la sustitucion de valores NA a 0
FileListIMG_corr <- list.files(path = dir.text, pattern = "corr.img$",
                               full.names = TRUE)
FileListIMG_corr

raster_corr <- list()
for (i in 1:length(FileListIMG_corr)) {
  raster_corr[[i]] <- raster(FileListIMG_corr[i]) 
}
raster_corr

# Sustitucion de valores NA
for (i in 1:length(FileListIMG_corr)){
  raster_corr[[i]][is.na(raster_corr[[i]])] <- 0
}

# Recorte de bordes por sustitucion con el poligono del area de estudio
for (i in 1:length(FileListIMG_corr)){
  raster_corr[[i]]<-mask(raster_corr[[i]], newarea)
}

# Escritura de los archivos
for (i in 1:length(FileListIMG_corr)){
writeRaster(raster_corr[[i]], 
            filename = file.path(dir.text, names(raster_corr[[i]])),
            overwrite =TRUE,
            format="HFA")
}


