# ---------------------------------------------------------------------------------------------------------------------
# Este script 1) Calcula la biomasa area por conglomerado de 1,600 m2 (4 parcelas de 400 m2)
#             2) Usa ecuasiones alometricas locales para calcular la biomasa por individuo
#             3) Revisa la Taxonomia (nombres de especies, corrije errores)
#             4) Extrae la densidad de la madera de bases de datos globales
#                o bien usa datos de densidad de la madera locales
# 
# Instalar los paquetes: 
# openxlsx, knitr, httr, BIOMASS, dplyr
# Usando la funcion de R install.packages()
#-----------------------------------------------------------------------------------------------------------------------
#
# PASO 1.- CREACI?N DE LA BASE DE DATOS
#-----------------------------------------------------------------------------------------------------------------------
# Establecer directorio de trabajo
setwd(choose.dir(caption = "Selecciona la carpeta de trabajo, debe contener los datos para el calculo de la biomasa"))
getwd()

# Abrir los paquetes, instalarlos si es necesario con install.packages("openxlsx")
library(openxlsx)
library(dplyr)

data_all <- read.xlsx(file.choose())

names(data_all)

# Crear lista de conglomerados
list<- data.frame(unique(data_all$Conglomerado))
list

# Identificar los conglomerados que contengan 4 parcelas
for (i in 1:length(list[,1])){
  list[i,2]<-length(unique((filter(data_all,
                                   Conglomerado == list[i,1]))$Sitio))==4
}
list

# Identificar en la base de datos los registros que no cumplen la condicion 
data_all$FLsit <- NA_character_
for(row in seq_len(nrow(list))){
  data_all$FLsit[substr(
    data_all$Conglomerado, 0, nchar(
      list$unique.data_all.Conglomerado.[row]) ) == list$unique.data_all.Conglomerado.[row] ] <- list$V2[row]
}

# Crear el dataframe con los registros que cumplen la condici?n de 4 sitios
data.list<- data_all[which(data_all$FLsit == TRUE),]

# Crear un nuevo data frame con las columnas ?tiles en el c?lculo de la biomasa
data<- data.list[ , c(1,3,8,9,11,13,18,21,22,49,50,51)]


# Dividir nombre cient?fico en g?nero y especie y agregarlo al dataframe en nuevas columnas
for (i in 1:(length(data$Conglomerado))){
   cad <- unlist( strsplit(data$NombreCientifico_APG[i], " "))
   data$GENERO[i] <- cad[1]
   data$ESPECIE[i] <- cad[2]
}

# Explorar los datos
names(data)
str(data)
head(data)

# PASO 2.- CORRECCIONES TAXONOMICAS
#-----------------------------------------------------------------------------------------------------------------------
# Abrir los siguientes paquetes 
library(BIOMASS)
library(knitr)
library(httr)

# Encontrar la sintaxis de cada especie usar la Formula: correctTaxo(genus, species)
Taxo<-correctTaxo(genus=data$GENERO, species=data$ESPECIE, useCache = T)

# Los nombres modificados se asignan como factores
Taxo$nameModified <- as.factor(Taxo$nameModified)
Taxo$speciesCorrected <- as.factor(Taxo$speciesCorrected)

# Se asignar los niveles de cada factor
levels(Taxo$nameModified)
levels(Taxo$speciesCorrected)

# Explorar el dataframe con los nombres corregidos
head(Taxo)

# Agregar los nombres de especies y generos correctos al dataframe
data$GENEROCor <- Taxo$genusCorrected
data$ESPECIECor <- Taxo$speciesCorrected

# Encontrar la familia a partir de los g?neros y ordenar si es necesario
# Para ello se usa la siguiente f?rmula: getTaxonomy(genus, findOrder = FALSE)
Fam<-getTaxonomy(data$GENEROCor, findOrder =F)

# Agregar la familia al dataframe
data$Familia <-Fam$family

names(data)

# PASO 3.- EXTRAER VALORES DE DENSIDAD DE LA MADERA USANDO BASES DE DATOS LOCALES Y GLOBALES
#-----------------------------------------------------------------------------------------------------------------------
# Formula syntax: getWoodDensity(genus, species, stand = NULL, family = NULL, region = "World", addWoodDensityData = NULL)
# Los requerimientos minimos en la formula son: Genus=, species=, stand=
# Cuando no existe un valor de densidad de la madera para una especie, se asigna el promedio de 
# la densidad de la madera a nivel de especies, g?nero o conglomerado.

# Cuando no solo disponemos de datos globales, se usan solo los de M?xico
# dataDM<-getWoodDensity(genus=data$GENEROCor,
#                       species=data$ESPECIECor,
#                       stand=data$Conglomarado,
#                       region="Mexico")

# En caso de contar con valores de densidad de la madera locales hacer lo siguiente
# Leer y explorar la base de datos local 
DMlocal <- read.table(choose.files(caption = "Selecciona el archivo con datos de DM"), header = TRUE, sep = ",")
head(DMlocal)

#formula cuando hay datos locales, ademas de los globales
dataDM<-getWoodDensity(genus=data$GENEROCor,
                       species=data$ESPECIECor,
                       stand=data$Conglomerado,
                       region="Mexico",
                       addWoodDensityData=DMlocal)

# Ver el n?mero de individuos con valores de densidad de la madera estimados a nivel de Especie, g?neros y parcela
# A nivel de especie 
sum(dataDM$levelWD=="species")
# a nivel de genero
sum(dataDM$levelWD=="genus")
# a nivel de parcela
sum(!dataDM$levelWD%in%c("genus","species"))

# Agregar la Densidad de la madera al dataframe
data$DM <- dataDM$meanWD
data$DM_Tipo <- dataDM$levelWD
data$DM_SD <- dataDM$sdWD
head(data)

# PASO 4.- C?LCULO DE LA BIOMASA A?REA PARA INDIVIDUOS 
#-----------------------------------------------------------------------------------------------------------------------
# Para el calculo de la biomasa de cada individuo se puede utilizar la funcion computeAGB(). 
# calcular AGB (Mg) por ?rbol ulizando la ecuaci?n alom?trica de Chave 2014
# Parametros: el di?mentro, la altura y la densidad de la madera
# AGBtree<-computeAGB(D=data$Diametro_normal,
#                    WD=data$DM,
#                    H =data$Altura_total)
# Calcular la biomasa en Mg por ha (o ton por ha) como sigue:
# Dividir la biomasa de cada individuo entre 1000, convirte kg a toneladas
# El conglomerado tiene 4 parcelas de 400 metros cuadrados = 1600 m2
# PARA CONVERTIR DE m2 a ha obtener el factor  10000/1600 = 6.25 y multiplicar
# data$AGB2 <- (AGBtree/1000)*6.25


# La segunda opciOn es calcular la biomasa con ecuasiones locales

# Definir par?metros para usar la funcion del c?lculo de biomasa
ID <- data$Conglomerado # Identificador de individuos
DAP <- data$Diametro_normal # DAP = Di?metro normal (cm)
H <-  data$Altura_total  # H = Altura Total (m)
FV <- data$Forma_Biologica_1 # FV = Forma de vida 
VSV <- data$Tipo_veg_SV # VSV = Vegetacion potencial registrada en la serie V de INEGI
VP <-  data$Veg_prim_levantada # VP = Vegetacion primaria definida por CONAFOR
Ds <- data$DM # Ds = Densidad de la madera
S <- 1600 # S = superficie en m2 de la parcela 4 parcelas de 400 m2

unique(FV)
#Funcion para el calculo de la  biomasa aerea por arbol en toneladas por Ha
AGBcalc <- function (DAP, H,FV, VSV, VP, Ds, S = NULL){
  
  #vegetacion definicion ----
  if (is.null(S)) {
    stop("Falta definir el valor de superficie del sitio de muestreo")
  }
  
  if (is.null(VP)) {
    VP = NA
  } else {
    VP = VP
  }
  arbol <- c("Arbol", "Arboles", "ND")
  lianas <- c("Liana", "Lianas", "Bejuco", "Bejucos")
  palmas <-c("Palma", "Palmas")
  if (!is.null(VSV)){
    ifelse (VSV %in% c("AGRICULTURA DE RIEGO", "AGRICULTURA DE TEMPORAL", "ASENTAMIENTOS HUMANOS", "MANGLAR",
                       "PALMAR NATURAL","PASTIZAL CULTIVADO", "PASTIZAL INDUCIDO", "SABANA",
                       "SIN VEGETACION APARENTE", "VEGETACION DE PETEN", "TULAR"), 
            vegetacion <- VP ,   vegetacion <- VSV)
    
  } else {
    vegetacion <- VP
  } 
  # Factor de conversion de superficie a 1 Ha, en esta caso s = 1600
  if  (is.null(S)){
    stop("S superficie del sitio de muestra no esta definido")
  } else {
    FS <- 10000 / S
  }
  
  # Calculo de la AGB por diferentes formas de vida con diferentes ecuaciones
  if (FV %in% palmas){
    #Ecuacion para palmas
    AGB <- ((-4.51+(7.7*H))/1000)*FS
    
  }
  else if  (FV %in% lianas) {
    #Ecuacion para lianas
    AGB <- ((exp(0.049+2.053*log(DAP)))/1000)*FS
    
  }
  else if (FV %in% arbol & (vegetacion %in% c("VM"))) {
    
    #Ecuacion para manglares obtenida de Chave 2003 
    AGB <- ((Ds*exp(-1.349+(1.980*log(DAP))+(0.207*(log(DAP))^2)-(0.281*(log(DAP))^3)))/1000)*FS
  }
  # Ecuaciones para arboles en selvas caducifolias y subcaducifolias
  else if  (FV %in% arbol & (
                        vegetacion %in% c("SELVA BAJA CADUCIFOLIA", "SELVA BAJA ESPINOSA CADUCIFOLIA", "SELVA BAJA SUBCADUCIFOLIA",
                                          "SELVA MEDIANA CADUCIFOLIA", "SELVA MEDIANA SUBCADUCIFOLIA",
                                          "SMS", "SBC", "SBS", "SMC", "SBK", "VW", "VS", "NA"))){
        ifelse (DAP < 10,
            # para arboles con dap < 10 se uso la ecuaci?n de Ramirez
            AGB <- ((exp(-4.1392+(0.99*log((DAP^2)*H))+(1.2268*Ds)))/1000)*FS,
            # para arboles con dap > 10 se uso la ecuaci?n de Chave 2005
            AGB <- ((Ds*exp(-0.667+(1.784*log(DAP))+(0.207*(log(DAP))^2)-(0.0281*(log(DAP))^3)))/1000)*FS )
  } 
  #Ecuaciones para arboles en selvas subperennifolias
  else if (FV %in% arbol & (
                       vegetacion %in% c("SELVA ALTA PERENNIFOLIA", "SELVA ALTA SUBPERENNIFOLIA","SELVA BAJA ESPINOSA SUBPERENNIFOLIA",
                                         "SELVA BAJA SUBPERENNIFOLIA", "SELVA MEDIANA SUBPERENNIFOLIA",
                                         "SMQ", "SBQ", "SAQ", "PT", "SBQP", "SMP", "PI", "SBP",
                                         "SAP", "ADV", "VPI", "VT", "VA", "RAS", "VPN", "SIN VEGETACION"))){
    
    ifelse (DAP < 10,
            #para arboles con dap < 10 se us? la ecuacian de Guyot
            AGB <- ((exp((1.3636*log(DAP))+(1.615*log(H))-2.9267))/1000)*FS,
            #para arboles con dap > 10 se us? la ecuacion de Cairns modificada por Urquiza-Haas
            AGB <- ((exp(-2.173+(0.868*log((DAP^2)*H))+(0.0939/2)*(Ds/0.7)))/1000)*FS)
  } else {
    AGB <- 0.0
  }
}

# Calculo de biomasa para cada registo
for (i in 1:(length(ID))){
  data$AGB[i] <- AGBcalc(DAP =  DAP[i],
                         H = H[i],
                         FV =  FV[i],
                         VSV =  VSV[i],
                         VP = VP[i],
                         Ds = Ds[i],
                         S = S)
}

  
# PASO 5.- CALCULO DE LA BIOMASA Y OTRAS VARIABLES POR CONGLOMEDARO
#-----------------------------------------------------------------------------------------------------------------------

# suma de la biomasa aerea por conglomerado
AGBconglomerado <- tapply(data$AGB, data$Conglomerado, FUN=sum)

# calculo del area basal AB por tallo en Ha
AB <- pi*(data$Diametro_normal/2)^2
data$AB <- (AB/10000)*6.25
# Obtener los sumatoria del area basal por conglomerado
ABconglomerado <- tapply(data$AB, data$Conglomerado, sum)

# Promedio del diametro, altura y densidad de la madera por conglomerado
data_promedios <- data %>% group_by(Conglomerado) %>%
  summarise_at(.vars = vars(DM, Diametro_normal ,Altura_total),
               .funs = c(mean="mean"), na.rm=T)

# Obtener las coordenadas de cada conglomerado, con las coordenadas de la parcela uno
data1 <- filter(data, Sitio == 1)
data2 <- data1 %>% group_by(Conglomerado) %>%
  summarise_at(.vars = vars(Anio,X,Y),
               .funs = c(mean="mean"), na.rm=T)

# Unir los dataframes
data_resumen <- as.data.frame(cbind(data2,data_promedios,ABconglomerado,AGBconglomerado))

#eliminar columas no deseadas
data_res<- data_resumen[ , c(1,2,3,4,7,8,9,10)]
head(data_res)
#Escribir un archivo CSV
write.csv(data_res, "FCP_bio_resumen.csv")
