# ---------------------------------------------------------------------------------------------------------------------
# Este script 1) Calcula el NDBI contruido con las capas HH Y HV pre-procesadas. Este indice intente
#                identificar la contribucin del volumen de dispersin de diferentes polarizaciones
#             2) Calculo de la textura de las bandas HH y HV asico como del NDBI, que contribuyen capturar 
#                variaciones en la estructura de la vegetacinonvierte 
# 
# Instalar los paquetes:
# glcm, rgeos, rgdal, 
# raster, sp, spatial.tools.
#------------------------------------------------------------------------------------------------------------------------
#
#
# PASO 1:  CONFIGURACION DEL DIRECTORIO DE TRABAJO Y LECTURA DE IMAGENES
#--------------------------------------------------------------------------------------------------------------------------

# cargar paquetes utilizados en este mdulo
library(glcm)        
library(rgeos)
library(rgdal)
library(raster)
library(sp)
library(spatial.tools)

##### Definir directorio de trabajo
setwd(choose.dir(default = "", 
                 caption = "Seleccionar folder con las imgenes en formato IMG"))
getwd()
folder <- getwd()

# cargar el shapefile del rea de estudio
area <- shapefile(choose.files(caption = "selecciona el shapefile del rea de estudio"))
plot(area)
summary(area)

# transformacin de coordenadas del polgono de inters similar a la proyeccin de los imgenes
newProj <- "+proj=lcc +lat_1=17.5 +lat_2=29.5 +lat_0=12 +lon_0=-102 +x_0=2500000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"

newarea<-  spTransform(area,newProj)
newarea

# Leer los archivos de cada polarizacin
FileListHH <- list.raster.files(path = folder, pattern = "HH(.*).img$",
                                return_bbox = FALSE,
                                return_rasters = TRUE)
FileListHH$raster_files
FileListHV <- list.raster.files(path = folder, pattern = "HV(.*).img$",
                                return_bbox = FALSE,
                                return_rasters = TRUE)
FileListHV$raster_files

# PASO 2: CALCULO DEL INDICE NDBI CON LAS CAPAS HH Y HV
#--------------------------------------------------------------------------------------------------------------------------
# NDBI = (HH - HV)/(HH + HV)
for (i in 1:length(FileListHH$raster_files)){
  overlay(FileListHH$raster_files[[i]],
          FileListHV$raster_files[[i]],
          # Definir la funcion
          fun=function (x,y) {return((x - y)/(x + y))},
          filename= paste(folder,"/","NDBI", sep = ""),
          overwrite =TRUE,
          format="HFA")
}


# PASO 3: CALCULO DE LAS MEDIDAS DE TEXTURA
#--------------------------------------------------------------------------------------------------------------------------
# Generar la lista de nombres de archivos a procesar 
FileListIMG <- list.files(path = folder, pattern = ".img$")
FileListIMG

# Crea lista de vectores para almacenar los resultados de las texturas
List <- vector(mode = "list",
               length = length(FileListIMG))
List

# clculo de texturas para todas las .img en la lista
for (i in 1:length(FileListIMG)){
  List[[i]] <- glcm(raster(FileListIMG[i]),
                    n_grey=64,
                    window = c(3,3),
                    shift=list(c(0,1), c(1,1), c(1,0), c(1,-1)),
                    statistics = c("mean", "variance", "homogeneity",
                                   "contrast", "dissimilarity", "entropy",
                                   "second_moment", "correlation"),
                    min_x=NULL, max_x=NULL,
                    na_opt="any", na_val=NA, 
                    scale_factor=1,
                    asinteger=FALSE)
}

#load("Texturas.RData")

for (i in 1:length(List)){
  plot(List[[i]])
}
#crear el directorio de salida de texturas
{Name.Dir.text <- "IMG_texturas"
{ifelse(!
          dir.exists(file.path(folder,Name.Dir.text)),
        dir.create(file.path(folder,Name.Dir.text)), FALSE
)
  dir.text <- file.path(folder,Name.Dir.text)}
}
#save.image(file = "TEXTURAS.RData")

#Guardar las imagenes de textura en formato HFA
for (i in 1:length(FileListIMG)){
  writeRaster(List[[i]]$glcm_mean,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_mean", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_variance,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$", 
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_vari", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_homogeneity,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_homo", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_contrast,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_cont", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_dissimilarity,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_diss", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_entropy,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$", 
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_entr", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_second_moment,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_seco", sep = ""),
              overwrite =TRUE,
              format="HFA")
  writeRaster(List[[i]]$glcm_correlation,
              paste(dir.text,"/",
                    sub(pattern = "(.*)\\..*$",
                        replacement = "\\1", basename(FileListIMG[i])),
                    "_corr", sep = ""),
              overwrite =TRUE,
              format="HFA")
}

# PASO 5: CORRECCION DE ARCCHIVOS
#--------------------------------------------------------------------------------------------------------------------------

#Lista de rster de salida 
FileListIMG_t <- list.raster.files(path = dir.text, pattern = ".img$", return_bbox = FALSE,
                                   return_rasters = TRUE)
FileListIMG_t

# Lista de textura correlation para la sustitucin de valores NA a 0
FileListIMG_corr <- list.raster.files(path = dir.text, pattern = "corr.img$", return_bbox = FALSE,
                                   return_rasters = TRUE)
FileListIMG_corr

# Sustitucin de valores NA
for (i in 1:length(FileListIMG_corr$raster_files)){
  FileListIMG_corr$raster_files[[i]][is.na(FileListIMG_corr$raster_files[[i]])] <- 0
}
# Recorte de bordes por sustitucion con el poligono del area de estudio
for (i in 1:length(FileListIMG_corr$raster_files)){
  FileListIMG_corr$raster_files[[i]]<-mask(FileListIMG_corr$raster_files[[i]], newarea)
}

# Escritura de los archivos
for (i in 1:length(FileListIMG_corr$raster_files)){
writeRaster(FileListIMG_corr$raster_files[[i]], 
            filename = file.path(dir.text, names(FileListIMG_corr$raster_files[[i]])),
            overwrite =TRUE,
            format="HFA")
}


