# ---------------------------------------------------------------------------------------------------------------------
# Este script 1) Crea un modelo para la estimacion de la biomasa utilizando datos de las imegenes (HH, HV, NDBI
#                y sus texturas) utilizando random forest (con el paquete ModelMap)
#             2) Valida el modelo con un conjunto de datos independientes
#             3) Utiliza el modelo de prediccion y las capas de datos para crear el mapa de biomasa
#             4) Obtener mapas de biomasa y de la incertidumbre para el area de bosque
# 
# Instalar los paquetes: 
# randomForest, ModelMap, raster,
# sp, spatial.tools, prettymapr, Metrics
# Usando la funcion de R install.packages()
#-----------------------------------------------------------------------------------------------------------------------
#
# PASO 1.- CONSTRUCCION DEL MODELO PARA ESTIMAR LA BIOMASA
#-----------------------------------------------------------------------------------------------------------------------
#seleccionar el directorio de trabajo
setwd(choose.dir(caption = "Seleccione el directorio RF")) 
folder <- getwd()

#Cargar paquetes que se utilizaran en este modulo
library(randomForest) 
library(ModelMap) 
library(raster)
library(sp)
library(spatial.tools)
library(prettymapr)
library(Metrics)

# Definicion de tipos de archivos 
qdatafn <- "DATOS.CSV" #archivo de datos
qdata.trainfn <- "DATOS_TRAIN.CSV" # archivo de datos para entrenar el modelo
qdata.testfn <- "DATOS_TEST.CSV" # archivo de datos para validar el modelo
 

#Separar el conjunto de datos en dos grupos: datos de entrenamiento y de validacion
#La proporcion de los datos de validacion es de un 30% y se seleccionan de manera aleatoria
get.test(proportion.test=0.3,
         qdatafn=qdatafn,
         seed=49,
         folder=folder,
         qdata.trainfn=qdata.trainfn,
         qdata.testfn=qdata.testfn)

# Definicion de los parametros del modelo
model.type <- "RF" # el Modelo es de Random Forest
MODELfn.a <- "AGB"

predList <- c( "HH_cont",   "HH_corr" ,  "HH_diss",  
               "HH_entr",   "HH_homo" ,  "HH_mean",
               "HH_seco",   "HH_vari", "HV_cont",
               "HV_corr",   "HV_diss",   "HV_entr",
               "HV_homo",   "HV_mean",   "HV_seco",  
               "HV_vari",   "NDBI_cont", "NDBI_corr",
               "NDBI_diss", "NDBI_entr", "NDBI_homo",
               "NDBI_mean", "NDBI_seco", "NDBI_vari",
               "HH",        "HV",        "NDBI" )

predFactor <- FALSE

response.name.a <-  "AGBconglomerado" # variable de respuesta
response.type <- "continuous"
seed.a <- 25
unique.rowname <- "Conglomerado"  #identificador unico de cada renglon 

# Construccion del modelo usando Random Forest
model.obj.AGB <- model.build( model.type=model.type,
                              qdata.trainfn=qdata.trainfn,
                              folder=folder,
                              unique.rowname=unique.rowname,
                              MODELfn=MODELfn.a,
                              predList=predList,
                              predFactor=predFactor,
                              response.name=response.name.a,
                              response.type=response.type,
                              seed=seed.a,
                              na.action = "na.omit")

model.obj.AGB


# PASO 2.- VALIDACION DEL MODELO PARA ESTIMAR BIOMASA
#-----------------------------------------------------------------------------------------------------------------------
# Validacion del Model usando un conjunto de datos independiente.
model.pred.AGB <- model.diagnostics( model.obj=model.obj.AGB,
                                     qdata.testfn=qdata.testfn,
                                     folder=folder,
                                     MODELfn=MODELfn.a,
                                     unique.rowname=unique.rowname,
                                     #parametros de la validacion del modelo
                                     prediction.type="TEST",
                                     device.type=c("pdf"),
                                     cex=1.2)


head(model.pred.AGB)

rmse.val <- rmse(model.pred.AGB$obs, model.pred.AGB$pred)
mean.obs <- mean(model.pred.AGB$obs)
percent.rmse <- (rmse.val / mean.obs) * 100
lm.Val <- lm(obs ~ pred , data = model.pred.AGB)

summary(lm.Val)

cbind(rmse.val,mean.obs,percent.rmse)

plot(x=model.pred.AGB$pred, y=model.pred.AGB$obs, xlab = "Predichos", ylab = "Observados",col="blue", lwd=1,xlim=c(0,300), ylim=c(0,300))
abline(lm.Val,col="blue", lwd=1.2, lty=2)
abline(0,1,col="red", lty=1, lwd=1)


# PASO 3.- HACER UN MAPA CON LA PREDICCION DE BIOMASA (TIPO RASTER)
#-----------------------------------------------------------------------------------------------------------------------


#Directorio donde se encuentran las imagenes usadas como predictores en el modelo
dir.IMG.bcks <- "C:/Mapeo_biomasa/FCP/IMG" #backscatters
dir.IMG.txtr <-  "C:/Mapeo_biomasa/FCP/IMG/IMG_texturas"#texturas

# Crear una lista con las imagenes de restrodispersion (Backscatter)
bcks <- list.raster.files(path = dir.IMG.bcks, pattern = ".img$", 
                          return_rasters = TRUE, return_bbox = FALSE)
bcks$raster_files[]

# crear un data frame de imagenes de backscatter
rastLUTfn.a <- data.frame()
for (i in 1: length(bcks$raster_files)){
  rastLUTfn.a[i,1]<- paste(dir.IMG.bcks,"/",
                          names(bcks$raster_files[[i]]),".img", sep = "")
  rastLUTfn.a[i,2]<- names(bcks$raster_files[[i]])
  rastLUTfn.a[i,3]<- 1 #bands number
}
rastLUTfn.a

# Crear una lista con las imagenes de textura 
txtr <- list.raster.files(path = dir.IMG.txtr, pattern = ".img$", 
                          return_rasters = TRUE, return_bbox = FALSE)
txtr$raster_files

# crear el data frame de las imagenes de textura
rastLUTfn.b <- data.frame()
for (i in 1: length(txtr$raster_files)){
  rastLUTfn.b[i,1]<- paste(dir.IMG.txtr,"/",
                           names(txtr$raster_files[[i]]),".img", sep = "")
  rastLUTfn.b[i,2]<- names(txtr$raster_files[[i]])
  rastLUTfn.b[i,3]<- 1
}
head(rastLUTfn.b)

# unir los dos dataframes  
rastLUTfn <- rbind(rastLUTfn.a, rastLUTfn.b)
head(rastLUTfn)


#crear el directorio de salida
{Name.Dir.MAPS <- "MAPS"
{ifelse(!
          dir.exists(file.path(folder,Name.Dir.MAPS)),
        dir.create(file.path(folder,Name.Dir.MAPS)), FALSE
)
  dir.MAPS <- file.path(folder,Name.Dir.MAPS)}
}

# Definir el nombre del mapa de salida
OUTPUTfn <-"AGB.img"
OUTPUTfn <- paste(dir.MAPS,OUTPUTfn,sep="/")

# Crear una imagen con el mapa predicho de biomasa
model.mapmake( model.obj=model.obj.AGB,
               folder=folder,		
               rastLUTfn=rastLUTfn,
               # parametros del mapa						
               OUTPUTfn=OUTPUTfn,
               MODELfn= MODELfn.a,
               map.sd = TRUE
)

mapgrid <- raster(OUTPUTfn)
plot(mapgrid)


#Crear una Mascara con areas de Bosque y areas de no Bosque y aplicarla al mapa
FNF <- raster(choose.files(caption = "select raster file FNF", multi = FALSE))

my_col <- c("darkgreen", "white", "blue")
plot (FNF, legend = TRUE, col = my_col)


mapgrid.mask <- mask(mapgrid,
                     FNF, maskvalue= 2)
mapgrid.mask <- mask(mapgrid.mask,
                FNF, maskvalue= 3)

plot(mapgrid.mask)

# Proyectar las coordenadas del mapa mascara a WGS84
mapgrid.a <- projectRaster(mapgrid.mask,
                         crs= "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs",
                         method = "ngb")

# guardar en disco el mapa creado
writeRaster(mapgrid.a,
            paste(dir.MAPS, "AGB_FCP.img", sep= "/"),
            overwrite= TRUE
            )

# PASO 4.- IMPRESION DE MAPAS
#-----------------------------------------------------------------------------------------------------------------------

# Definir la paleta de colores 
l <- seq(100,0,length.out=101)
c <- seq(0,100,length.out=101)
col.ramp <- hcl(h = 120, c = c, l = l)

r.range <- c(minValue(mapgrid.a), maxValue(mapgrid.a))

# Imprimir 
plot(mapgrid.a,
     col = col.ramp,
     main = response.name.a,
     zlim = r.range)

# Generar mapa de incertidumbre
coefv <- raster(choose.files(caption="Seleccionar raster Coeficiente de Variacion"))

coefv.mask <- mask(coefv,
                     FNF, maskvalue= 2)
coefv.mask <- mask(coefv.mask,
                     FNF, maskvalue= 3)

# conversion a WGS84
coefv.a <- projectRaster(coefv.mask,
                           crs= "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs",
                           method = "ngb")

ramp <- colorRampPalette(c("green4","yellow"))
plot(coefv.a, breaks = c(0,0.25,0.50,0.75,1, round(maxValue(coefv.a),2)),
     col=terrain.colors(6))

writeRaster(coefv.a,
            paste(dir.MAPS, "AGB_FCP_coefv.img", sep= "/"),
            overwrite= TRUE
)

